// yapsnmp 0.7.8
//
// SWIG interface file exposing most functions of libsnmp from net-snmp
//
// Wrapper functions to enable access from Python and to conform to a Python
// style of programming are also made available.
//
//  -- Yves Perrenoud <yves@xpand.org>

%module netsnmp

// A few type definitions to clarify what to do

typedef unsigned long size_t
typedef char u_char
typedef unsigned long u_long
typedef unsigned short u_short

// Includes for both the C code and to generate the interface

%{
#include <net-snmp/net-snmp-config.h>
#include <net-snmp/net-snmp-includes.h>
#include <net-snmp/library/transform_oids.h>
#include <net-snmp/library/snmp_api.h>
#include <net-snmp/library/mib.h>
%}

%include "asn1.h"
%include "snmp_api.h"
%include "snmp.h"
%include "snmp_client.h"
%include "mib.h"
%include "default_store.h"

// allow the instantiation and destruction of this structure

%addmethods snmp_session {
  snmp_session() {
    return (struct snmp_session *) malloc(sizeof(struct snmp_session));
  };
  ~snmp_session() {
    free(self);
  };
};

// A wrapper function and structure to allow the parsing of pointers
// I'm assuming my destruct method will allow recycling and avoid memory
// leaks, but I could be wrong... may have to use typemaps

%{
  struct pyoid {
    oid oid[MAX_OID_LEN];
    size_t len;
  };
%}

struct pyoid {
  oid oid[MAX_OID_LEN];
  size_t len;
  %addmethods {
    ~pyoid() {
      free(self);
    }
  }
};

%inline %{
  struct pyoid *py_get_node(const char *string) {
    int status;

    struct pyoid *pyoid = (struct pyoid *) malloc(sizeof(struct pyoid));
    pyoid->len = MAX_OID_LEN;

    Py_BEGIN_ALLOW_THREADS
    status = get_node(string, pyoid->oid, &pyoid->len);
    Py_END_ALLOW_THREADS

    if (status == 1)
      return pyoid;
    else
      return 0;
  }
%}

%inline %{
  struct pyoid *py_read_objid(const char *string) {
    int status;

    struct pyoid *pyoid = (struct pyoid *) malloc(sizeof(struct pyoid));
    pyoid->len = MAX_OID_LEN;

    Py_BEGIN_ALLOW_THREADS
    status = read_objid(string, pyoid->oid, &pyoid->len);
    Py_END_ALLOW_THREADS

    if (status == 1)
      return pyoid;
    else
      return 0;
  }
%}

// just a wrapper around the fact that synch_response wants a double pointer

%inline %{
  netsnmp_pdu *py_snmp_sess_synch_response(void *session, netsnmp_pdu *pdu) {
    netsnmp_pdu *response;
    int rc;

    Py_BEGIN_ALLOW_THREADS
    rc = snmp_sess_synch_response(session, pdu, &response);
    Py_END_ALLOW_THREADS

    if (rc == 0)
      return response;
    else
      return 0;
  }
%}

// These typemaps are there to support the clean (no memory leak) return of a
// string to python

%typemap(python,ignore) char *OUTPUT { ; }

%typemap(python,argout) char *OUTPUT {
  $target = PyString_FromString($source);
}

%typemap(python,freearg) char *OUTPUT {
  free($source);
}

// first use of the typemap on the wrapper around snmp_error

%apply char *OUTPUT { char *buf };

%inline %{
  void py_snmp_sess_error(void *session, char *buf) {
    Py_BEGIN_ALLOW_THREADS
    snmp_sess_error(session, NULL, NULL, &buf);
    Py_END_ALLOW_THREADS
  }
%}

// let's use this typemap on the wrapper around sprint_value

%typemap(python,check) char *OUTPUT {
  $target = (char *) malloc(SPRINT_MAX_LEN);
}

%apply char *OUTPUT { char *buf };

%inline %{
  void py_sprint_value (struct variable_list *vars, char *buf) {
    Py_BEGIN_ALLOW_THREADS
    snprint_value(buf, SPRINT_MAX_LEN, vars->name, vars->name_length, vars);
    Py_END_ALLOW_THREADS
  }
%}

%inline %{
  void py_sprint_variable (struct variable_list *vars, char *buf) {
    Py_BEGIN_ALLOW_THREADS
    snprint_variable(buf, SPRINT_MAX_LEN, vars->name, vars->name_length, vars);
    Py_END_ALLOW_THREADS
  }
%}

%inline %{
  void py_sprint_objid (struct pyoid *oid, char *buf) {
    Py_BEGIN_ALLOW_THREADS
    snprint_objid(buf, SPRINT_MAX_LEN, oid->oid, oid->len);
    Py_END_ALLOW_THREADS
  }
%}
